#!/usr/bin/env python

#   The gedit XML Tools plugin provides many useful tools for xml development.
#   Copyright (C) 2008  Pascal Sachs
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from lxml import etree
from os.path import realpath, join, dirname

class XMLToolsXmlProcessor:
	def __init__(self, xml_file):
		try:
			self.__error_list = []
			self.xml = etree.parse(xml_file)
			path = dirname(realpath(xml_file))
			schema_file = join(path, self.xml.getroot().values()[0])
			self.xml_schema = etree.XMLSchema(etree.parse(schema_file))
			self.valid = self.__validate()
		except etree.XMLSyntaxError, e:
			log = e.error_log
			for error in log:
				self.__error_list.append((error.line, error.message))
			self.valid = False
		except:
			self.__error_list.append((0, "unknown error"))
			self.valid = False
	
	def __validate(self):
		try:
			self.xml_schema.assertValid(self.xml)
			self.valid = True
			return True
		except:	
			log = self.xml_schema.error_log
			for error in log:
				self.__error_list.append((error.line, error.message))
			self.valid = False
			return False
	
	def xpath(self, xpath_query):
		result = ""
		try:
			xRes = self.xml.xpath(xpath_query)
			for x in xRes:
				result += etree.tostring(x) + "\n"
		except:
			result = "XPath syntax error\n"
		return	result

	def update_document(self, new_xml = None):
		try:
			if new_xml != None:
				self.xml_file = new_xml
			self.xml = etree.parse(self.xml_file)
			path = dirname(realpath(new_xml))
			schema_file = join(path, self.xml.getroot().values()[0])
			self.xml_schema = etree.XMLSchema(etree.parse(schema_file))
			self.valid = self.validate()
		except etree.XMLSyntaxError, e:
			log = e.error_log
			for error in log:
				self.__error_list.append((error.line, error.message))
			self.valid = False
		except:
			self.__error_list.append((0, "unknown error"))
			self.valid = False
	
	def get_error(self):
		return self.__error_list
	
